@@ -4,7 +4,7 @@
  * Copyright (C) 2003 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -36,6 +36,7 @@ G_BEGIN_DECLS
 #define EVC_ENCODING        "ENCODING"
 #define EVC_FBURL           "FBURL"
 #define EVC_FN              "FN"
+#define EVC_GEO		    "GEO"
 #define EVC_ICSCALENDAR     "ICSCALENDAR" /* XXX should this be X-EVOLUTION-ICSCALENDAR? */
 #define EVC_KEY             "KEY"
 #define EVC_LABEL           "LABEL"
@@ -85,7 +86,9 @@ G_BEGIN_DECLS
 #define EVC_X_VIDEO_URL     	"X-EVOLUTION-VIDEO-URL"
 #define EVC_X_WANTS_HTML    	"X-MOZILLA-HTML"
 #define EVC_X_YAHOO         	"X-YAHOO"
+#define EVC_X_GADUGADU        "X-GADUGADU"
 #define EVC_X_GROUPWISE     	"X-GROUPWISE"
+#define EVC_X_BOOK_URI     	"X-EVOLUTION-BOOK-URI"
 
 typedef enum {
 	EVC_FORMAT_VCARD_21,
@@ -138,19 +141,22 @@ void    e_vcard_dump_structure               (EVCard *evc);
 EVCardAttribute *e_vcard_attribute_new               (const char *attr_group, const char *attr_name);
 void             e_vcard_attribute_free              (EVCardAttribute *attr);
 EVCardAttribute *e_vcard_attribute_copy              (EVCardAttribute *attr);
-void             e_vcard_remove_attributes           (EVCard *evcard, const char *attr_group, const char *attr_name);
-void             e_vcard_remove_attribute            (EVCard *evcard, EVCardAttribute *attr);
-void             e_vcard_add_attribute               (EVCard *evcard, EVCardAttribute *attr);
+void             e_vcard_remove_attributes           (EVCard *evc, const char *attr_group, const char *attr_name);
+void             e_vcard_remove_attribute            (EVCard *evc, EVCardAttribute *attr);
+void             e_vcard_add_attribute               (EVCard *evc, EVCardAttribute *attr);
 void             e_vcard_add_attribute_with_value    (EVCard *evcard, EVCardAttribute *attr, const char *value);
 void             e_vcard_add_attribute_with_values   (EVCard *evcard, EVCardAttribute *attr, ...);
 void             e_vcard_attribute_add_value         (EVCardAttribute *attr, const char *value);
 void             e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, int len);
 void             e_vcard_attribute_add_values        (EVCardAttribute *attr, ...);
+void             e_vcard_attribute_remove_value      (EVCardAttribute *attr, const char *s);
 void             e_vcard_attribute_remove_values     (EVCardAttribute *attr);
 void             e_vcard_attribute_remove_params     (EVCardAttribute *attr);
+void             e_vcard_attribute_remove_param      (EVCardAttribute *attr, const char *param_name);
+void             e_vcard_attribute_remove_param_value (EVCardAttribute *attr, const char *param_name, const char *s);
 
 /* attribute parameters */
-EVCardAttributeParam* e_vcard_attribute_param_new             (const char *param_name);
+EVCardAttributeParam* e_vcard_attribute_param_new             (const char *name);
 void                  e_vcard_attribute_param_free            (EVCardAttributeParam *param);
 EVCardAttributeParam* e_vcard_attribute_param_copy            (EVCardAttributeParam *param);
 void                  e_vcard_attribute_add_param             (EVCardAttribute *attr, EVCardAttributeParam *param);
@@ -167,6 +173,7 @@ void                  e_vcard_attribute_param_remove_values   (EVCardAttributePa
 
 /* EVCard* accessors.  nothing returned from these functions should be
    freed by the caller. */
+EVCardAttribute *e_vcard_get_attribute        (EVCard *evc, const char *name);
 GList*           e_vcard_get_attributes       (EVCard *evcard);
 const char*      e_vcard_attribute_get_group  (EVCardAttribute *attr);
 const char*      e_vcard_attribute_get_name   (EVCardAttribute *attr);
@@ -179,6 +186,7 @@ char*                 e_vcard_attribute_get_value             (EVCardAttribute *
 GString*              e_vcard_attribute_get_value_decoded     (EVCardAttribute *attr);
 
 GList*           e_vcard_attribute_get_params       (EVCardAttribute *attr);
+GList*           e_vcard_attribute_get_param        (EVCardAttribute *attr, const char *name);
 const char*      e_vcard_attribute_param_get_name   (EVCardAttributeParam *param);
 GList*           e_vcard_attribute_param_get_values (EVCardAttributeParam *param);
 
@@ -186,8 +194,8 @@ GList*           e_vcard_attribute_param_get_values (EVCardAttributeParam *param
 gboolean         e_vcard_attribute_has_type         (EVCardAttribute *attr, const char *typestr);
 
 /* Utility functions. */
-char*            e_vcard_escape_string (const char *str);
-char*            e_vcard_unescape_string (const char *str);
+char*            e_vcard_escape_string (const char *s);
+char*            e_vcard_unescape_string (const char *s);
 
 G_END_DECLS
 
