@@ -23,9 +23,9 @@
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-main.h>
-#include <libedataserver/e-url.h>
-#include <libedataserver/e-source.h>
-#include <libedataserver/e-data-server-module.h>
+#include "libedataserver/e-url.h"
+#include "libedataserver/e-source.h"
+#include "libedataserver/e-data-server-module.h"
 #include "e-cal-backend.h"
 #include "e-data-cal.h"
 #include "e-data-cal-factory.h"
@@ -86,39 +86,13 @@ get_backend_factory (GHashTable *methods, const char *method, icalcomponent_kind
 	
 	kinds = g_hash_table_lookup (methods, method);
 	if (!kinds)
-		return 0;
+		return NULL;
 
 	factory = g_hash_table_lookup (kinds, GINT_TO_POINTER (kind));
 
 	return factory;
 }
 
-/* Looks up a calendar backend in a factory's hash table of uri->cal.  If
- * *non-NULL, orig_uri_return will be set to point to the original key in the
- * *hash table.
- */
-static ECalBackend *
-lookup_backend (EDataCalFactory *factory, const char *uristr)
-{
-	EDataCalFactoryPrivate *priv;
-	EUri *uri;
-	ECalBackend *backend;
-	char *tmp;
-
-	priv = factory->priv;
-
-	uri = e_uri_new (uristr);
-	if (!uri)
-		return NULL;
-
-	tmp = e_uri_to_string (uri, FALSE);
-	backend = g_hash_table_lookup (priv->backends, tmp);
-	g_free (tmp);
-	e_uri_free (uri);
-
-	return backend;
-}
-
 /* Callback used when a backend loses its last connected client */
 static void
 backend_last_client_gone_cb (ECalBackend *backend, gpointer data)
@@ -140,7 +114,7 @@ backend_last_client_gone_cb (ECalBackend *backend, gpointer data)
 	g_assert (uristr != NULL);
 	uri = g_strdup_printf("%s:%d", uristr, (int)e_cal_backend_get_kind(backend));
 
-	ret_backend = lookup_backend (factory, uri);
+	ret_backend = g_hash_table_lookup (factory->priv->backends, uri);
 	g_assert (ret_backend != NULL);
 	g_assert (ret_backend == backend);
 
@@ -164,6 +138,7 @@ impl_CalFactory_getCal (PortableServer_Servant servant,
 			const GNOME_Evolution_Calendar_CalListener listener,
 			CORBA_Environment *ev)
 {
+	GNOME_Evolution_Calendar_Cal ret_cal = NULL;
 	EDataCalFactory *factory;
 	EDataCalFactoryPrivate *priv;
 	EDataCal *cal = CORBA_OBJECT_NIL;
@@ -227,7 +202,7 @@ impl_CalFactory_getCal (PortableServer_Servant servant,
 	CORBA_exception_free (&ev2);
 
 	/* Look for an existing backend */
-	backend = lookup_backend (factory, uri_type_string);
+	backend = g_hash_table_lookup (factory->priv->backends, uri_type_string);
 	if (!backend) {
 		/* There was no existing backend, create a new one */
 		backend = e_cal_backend_factory_new_backend (backend_factory, source);
@@ -259,12 +234,13 @@ impl_CalFactory_getCal (PortableServer_Servant servant,
 	e_cal_backend_add_client (backend, cal);
 	e_cal_backend_set_mode (backend, priv->mode);
 	
+	ret_cal = CORBA_Object_duplicate (BONOBO_OBJREF (cal), ev);
  cleanup:
 	e_uri_free (uri);
 	g_free (uri_type_string);
 	g_object_unref (source);
 
-	return CORBA_Object_duplicate (BONOBO_OBJREF (cal), ev);
+	return ret_cal;
 }
 
 
