@@ -22,31 +22,29 @@
  *
  */
 
-/* (from glibc headers:
-   POSIX says that <sys/types.h> must be included (by the caller) before <regex.h>.  */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+/* POSIX requires <sys/types.h> be included before <regex.h> */
 #include <sys/types.h>
-#include <regex.h>
-#include <string.h>
+
 #include <ctype.h>
+#include <regex.h>
 #include <stdio.h>
+#include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 
-#include "libedataserver/e-sexp.h"
+#include <libedataserver/e-sexp.h>
 
 #include "camel-exception.h"
-#include "camel-i18n.h"
 #include "camel-mime-message.h"
 #include "camel-multipart.h"
 #include "camel-search-private.h"
 #include "camel-stream-mem.h"
 
-
 #define d(x)
 
 static inline guint32
@@ -250,7 +248,7 @@ camel_ustrstrcase (const char *haystack, const char *needle)
 	
 	puni = nuni = g_alloca (sizeof (gunichar) * strlen (needle));
 	
-	p = needle;
+	p = (const unsigned char *) needle;
 	while ((u = camel_utf8_getc(&p)))
 		*puni++ = g_unichar_tolower (u);
 	
@@ -305,8 +303,8 @@ camel_ustrcasecmp (const char *s1, const char *s2)
 	
 	CAMEL_SEARCH_COMPARE (s1, s2, NULL);
 	
-	u1 = camel_utf8_getc((const unsigned char **)&s1);
-	u2 = camel_utf8_getc((const unsigned char **)&s2);
+	u1 = camel_utf8_getc(&s1);
+	u2 = camel_utf8_getc(&s2);
 	while (u1 && u2) {
 		u1 = g_unichar_tolower (u1);
 		u2 = g_unichar_tolower (u2);
@@ -315,8 +313,8 @@ camel_ustrcasecmp (const char *s1, const char *s2)
 		else if (u1 > u2)
 			return 1;
 		
-		u1 = camel_utf8_getc((const unsigned char **)&s1);
-		u2 = camel_utf8_getc((const unsigned char **)&s2);
+		u1 = camel_utf8_getc(&s1);
+		u2 = camel_utf8_getc(&s2);
 	}
 	
 	/* end of one of the strings ? */
@@ -335,8 +333,8 @@ camel_ustrncasecmp (const char *s1, const char *s2, size_t len)
 	
 	CAMEL_SEARCH_COMPARE (s1, s2, NULL);
 	
-	u1 = camel_utf8_getc((const unsigned char **)&s1);
-	u2 = camel_utf8_getc((const unsigned char **)&s2);
+	u1 = camel_utf8_getc(&s1);
+	u2 = camel_utf8_getc(&s2);
 	while (len > 0 && u1 && u2) {
 		u1 = g_unichar_tolower (u1);
 		u2 = g_unichar_tolower (u2);
@@ -346,8 +344,8 @@ camel_ustrncasecmp (const char *s1, const char *s2, size_t len)
 			return 1;
 		
 		len--;
-		u1 = camel_utf8_getc((const unsigned char **)&s1);
-		u2 = camel_utf8_getc((const unsigned char **)&s2);
+		u1 = camel_utf8_getc(&s1);
+		u2 = camel_utf8_getc(&s2);
 	}
 	
 	if (len == 0)
@@ -381,7 +379,7 @@ header_match(const char *value, const char *match, camel_search_match_t how)
 	/* from dan the man, if we have mixed case, perform a case-sensitive match,
 	   otherwise not */
 	p = (const unsigned char *)match;
-	while (c = camel_utf8_getc(&p)) {
+	while ((c = camel_utf8_getc(&p))) {
 		if (g_unichar_isupper(c)) {
 			switch (how) {
 			case CAMEL_SEARCH_MATCH_EXACT:
@@ -427,7 +425,7 @@ camel_search_header_match (const char *value, const char *match, camel_search_ma
 	gunichar c;
 
 	ptr = value;
-	while ((c = camel_utf8_getc((const unsigned char **)&ptr)) && g_unichar_isspace(c))
+	while ((c = camel_utf8_getc(&ptr)) && g_unichar_isspace(c))
 		value = ptr;
 	
 	switch(type) {
@@ -512,7 +510,7 @@ camel_search_message_body_contains (CamelDataWrapper *object, regex_t *pattern)
 		
 		camel_data_wrapper_write_to_stream (containee, CAMEL_STREAM (mem));
 		camel_stream_write (CAMEL_STREAM (mem), "", 1);
-		truth = regexec (pattern, mem->buffer->data, 0, NULL, 0) == 0;
+		truth = regexec (pattern, (char *) mem->buffer->data, 0, NULL, 0) == 0;
 		camel_object_unref (mem);
 	}
 	
@@ -624,14 +622,14 @@ camel_search_words_simple(struct _camel_search_words *wordin)
 			word->word = g_strdup(wordin->words[i]->word);
 			g_ptr_array_add(list, word);
 		} else {
-			ptr = wordin->words[i]->word;
+			ptr = (const unsigned char *) wordin->words[i]->word;
 			start = last = ptr;
 			do {
 				c = camel_utf8_getc(&ptr);
 				if (c == 0 || !g_unichar_isalnum(c)) {
 					if (last > start) {
 						word = g_malloc0(sizeof(*word));
-						word->word = g_strndup(start, last-start);
+						word->word = g_strndup((gchar *) start, last-start);
 						word->type = type;
 						g_ptr_array_add(list, word);
 						all |= type;
