@@ -43,20 +43,20 @@
 
 #include <errno.h>
 
-#include <camel/camel-exception.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-data-wrapper.h>
+#include <glib.h>
+#include <glib/gi18n-lib.h>
 
-#include <camel/camel-mime-part.h>
-#include <camel/camel-multipart-signed.h>
-#include <camel/camel-stream-fs.h>
-#include <camel/camel-stream-filter.h>
-#include <camel/camel-mime-filter-basic.h>
-#include <camel/camel-mime-filter-canon.h>
-
-#include "camel-smime-context.h"
+#include "camel-data-wrapper.h"
+#include "camel-exception.h"
+#include "camel-mime-filter-basic.h"
+#include "camel-mime-filter-canon.h"
+#include "camel-mime-part.h"
+#include "camel-multipart-signed.h"
 #include "camel-operation.h"
-#include "camel-i18n.h"
+#include "camel-smime-context.h"
+#include "camel-stream-filter.h"
+#include "camel-stream-fs.h"
+#include "camel-stream-mem.h"
 
 #define d(x)
 
@@ -134,6 +134,9 @@ camel_smime_context_describe_part(CamelSMIMEContext *context, CamelMimePart *par
 	CamelContentType *ct;
 	const char *tmp;
 
+	if (!part)
+		return flags;
+
 	ct = camel_mime_part_get_content_type(part);
 
 	if (camel_content_type_is(ct, "multipart", "signed")) {
@@ -667,7 +670,7 @@ sm_verify(CamelCipherContext *context, CamelMimePart *ipart, CamelException *ex)
 	NSSCMSDecoderContext *dec;
 	NSSCMSMessage *cmsg;
 	CamelStreamMem *mem;
-	CamelStream *constream;
+	CamelStream *constream = NULL;
 	CamelCipherValidity *valid = NULL;
 	CamelContentType *ct;
 	const char *tmp;
