@@ -28,13 +28,15 @@
 #endif
 
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+
+#include <glib.h>
+#include <glib/gi18n-lib.h>
 
 #include "camel-debug.h"
 #include "camel-exception.h"
 #include "camel-folder.h"
-#include "camel-i18n.h"
 #include "camel-private.h"
 #include "camel-session.h"
 #include "camel-store.h"
@@ -140,7 +142,7 @@ camel_store_init (void *o)
 	store->mode = CAMEL_STORE_READ|CAMEL_STORE_WRITE;
 
 	store->priv = g_malloc0 (sizeof (*store->priv));
-	store->priv->folder_lock = e_mutex_new (E_MUTEX_REC);
+	g_static_rec_mutex_init (&store->priv->folder_lock);
 }
 
 static void
@@ -151,7 +153,7 @@ camel_store_finalize (CamelObject *object)
 	if (store->folders)
 		camel_object_bag_destroy(store->folders);
 	
-	e_mutex_destroy (store->priv->folder_lock);
+	g_static_rec_mutex_free (&store->priv->folder_lock);
 	
 	g_free (store->priv);
 }
@@ -289,7 +291,7 @@ camel_store_get_folder (CamelStore *store, const char *folder_name, guint32 flag
 	if (camel_debug_start(":store")) {
 		char *u = camel_url_to_string(((CamelService *)store)->url, CAMEL_URL_HIDE_PASSWORD);
 
-		printf("CamelStore('%s'):get_folder('%s', %d) = %p\n", u, folder_name, flags, folder);
+		printf("CamelStore('%s'):get_folder('%s', %u) = %p\n", u, folder_name, flags, folder);
 		if (ex && ex->id)
 			printf("  failed: '%s'\n", ex->desc);
 		g_free(u);
@@ -472,7 +474,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_namein, const char
 				&& strncmp(folder->full_name, old_name, oldlen) == 0
 				&& folder->full_name[oldlen] == '/')) {
 				d(printf("Found subfolder of '%s' == '%s'\n", old_name, folder->full_name));
-				CAMEL_FOLDER_LOCK(folder, lock);
+				CAMEL_FOLDER_REC_LOCK(folder, lock);
 			} else {
 				g_ptr_array_remove_index_fast(folders, i);
 				i--;
@@ -499,7 +501,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_namein, const char
 			camel_folder_rename(folder, new);
 			g_free(new);
 
-			CAMEL_FOLDER_UNLOCK(folder, lock);
+			CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 			camel_object_unref(folder);
 		}
 
@@ -517,7 +519,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_namein, const char
 		/* Failed, just unlock our folders for re-use */
 		for (i=0;i<folders->len;i++) {
 			folder = folders->pdata[i];
-			CAMEL_FOLDER_UNLOCK(folder, lock);
+			CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 			camel_object_unref(folder);
 		}
 	}
@@ -989,6 +991,10 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 	tail = top;
 	for (i = 0; i < folders->len; i++) {
 		fi = folders->pdata[i];
+
+		if (fi->child)
+			fi->flags &= ~CAMEL_FOLDER_NOCHILDREN;
+
 		if (fi->parent || fi == top)
 			continue;
 		if (tail == NULL) {
