@@ -29,33 +29,39 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h> 
 #endif
+
 #include <errno.h>
+#include <string.h>
 #include <time.h>
-#include <libedataserver/e-msgport.h>
-#include "camel-groupwise-folder.h"
-#include "camel-groupwise-store.h"
-#include "camel-folder.h"
-#include "camel-folder-search.h"
-#include "camel-groupwise-store.h"
-#include "camel-groupwise-summary.h"
-#include "camel-groupwise-utils.h"
-#include "camel-i18n.h" 
-#include "camel-private.h"
-#include "camel-groupwise-private.h"
-#include "camel-groupwise-journal.h"
-#include "camel-groupwise-utils.h"
-#include "camel-stream-mem.h"
-#include "camel-string-utils.h"
-#include "camel-session.h"
-#include <e-gw-connection.h>
-#include <e-gw-item.h>
+#include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
+#include <glib.h>
+#include <glib/gi18n-lib.h>
 
 #include <libsoup/soup-misc.h>
 
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <libedataserver/e-msgport.h>
+
+#include <e-gw-connection.h>
+#include <e-gw-item.h>
+
+#include "camel-folder-search.h"
+#include "camel-folder.h"
+#include "camel-private.h"
+#include "camel-session.h"
+#include "camel-stream-mem.h"
+#include "camel-string-utils.h"
+
+#include "camel-groupwise-folder.h"
+#include "camel-groupwise-journal.h"
+#include "camel-groupwise-private.h"
+#include "camel-groupwise-store.h"
+#include "camel-groupwise-store.h"
+#include "camel-groupwise-summary.h"
+#include "camel-groupwise-utils.h"
+#include "camel-groupwise-utils.h"
 
 #define ADD_JUNK_ENTRY 1
 #define REMOVE_JUNK_ENTRY -1
@@ -69,8 +75,8 @@ static CamelOfflineFolderClass *parent_class = NULL;
 struct _CamelGroupwiseFolderPrivate {
 
 #ifdef ENABLE_THREADS
-	EMutex *search_lock;    /* for locking the search object */
-	EMutex *cache_lock;     /* for locking the cache object */
+	GStaticMutex search_lock;	/* for locking the search object */
+	GStaticRecMutex cache_lock;	/* for locking the cache object */
 #endif
 
 };
@@ -183,11 +189,8 @@ groupwise_folder_get_message( CamelFolder *folder, const char *uid, CamelExcepti
 	if (msg)
 		camel_medium_set_header (CAMEL_MEDIUM (msg), "X-Evolution-Source", groupwise_base_url_lookup (priv));
 
-	if(!strcmp (folder->full_name, "Sent Items")) 
-		goto end; /*Dont cache if its sent items, since we need to Track Status*/
-
 	/* add to cache */
-	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 	if ((cache_stream = camel_data_cache_add (gw_folder->cache, "cache", uid, NULL))) {
 		if (camel_data_wrapper_write_to_stream ((CamelDataWrapper *) msg, cache_stream) == -1
 				|| camel_stream_flush (cache_stream) == -1)
@@ -195,9 +198,8 @@ groupwise_folder_get_message( CamelFolder *folder, const char *uid, CamelExcepti
 		camel_object_unref (cache_stream);
 	}
 
-	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 
-end:
 	camel_message_info_free (&mi->info);
 	g_free (container_id);
 	return msg;
@@ -351,6 +353,8 @@ groupwise_msg_set_recipient_list (CamelMimeMessage *msg, EGwItem *item)
 
 			}
 		}
+
+		/* README: This entire status-opt code should go away once the new status-tracking code is tested */
 		if (enabled) {
 			camel_medium_add_header ( CAMEL_MEDIUM (msg), "X-gw-status-opt", (const char *)status_opt);
 			g_free (status_opt);
@@ -415,10 +419,10 @@ groupwise_folder_rename (CamelFolder *folder, const char *new)
 
 	summary_path = g_strdup_printf ("%s/summary", folder_dir);
 
-	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 	g_free (gw_folder->cache->path);
 	gw_folder->cache->path = g_strdup (folder_dir);
-	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 
 	((CamelFolderClass *)parent_class)->rename(folder, new);
 	camel_folder_summary_set_filename (folder->summary, summary_path);
@@ -437,7 +441,7 @@ groupwise_folder_search_by_expression (CamelFolder *folder, const char *expressi
 	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER(folder);
 	GPtrArray *matches;
 
-	CAMEL_GROUPWISE_FOLDER_LOCK(folder, search_lock);
+	CAMEL_GROUPWISE_FOLDER_LOCK(gw_folder, search_lock);
 	camel_folder_search_set_folder (gw_folder->search, folder);
 	matches = camel_folder_search_search(gw_folder->search, expression, NULL, ex);
 	CAMEL_GROUPWISE_FOLDER_UNLOCK(gw_folder, search_lock);
@@ -454,7 +458,7 @@ groupwise_folder_search_by_uids(CamelFolder *folder, const char *expression, GPt
 	if (uids->len == 0)
 		return g_ptr_array_new();
 
-	CAMEL_GROUPWISE_FOLDER_LOCK(folder, search_lock);
+	CAMEL_GROUPWISE_FOLDER_LOCK(gw_folder, search_lock);
 
 	camel_folder_search_set_folder(gw_folder->search, folder);
 	matches = camel_folder_search_search(gw_folder->search, expression, uids, ex);
@@ -471,7 +475,7 @@ groupwise_folder_search_free (CamelFolder *folder, GPtrArray *uids)
 
 	g_return_if_fail (gw_folder->search);
 
-	CAMEL_GROUPWISE_FOLDER_LOCK(folder, search_lock);
+	CAMEL_GROUPWISE_FOLDER_LOCK(gw_folder, search_lock);
 
 	camel_folder_search_free_result (gw_folder->search, uids);
 
@@ -614,16 +618,16 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 	container_id =  camel_groupwise_store_container_id_lookup (gw_store, folder->full_name) ;
 
-	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 	if (!camel_groupwise_store_connected (gw_store, ex)) {
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 		camel_exception_clear (ex);
 		return;
 	}
-	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 
 	count = camel_folder_summary_count (folder->summary);
-	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 	for (i=0 ; i < count ; i++) {
 		guint32 flags = 0;
 		info = camel_folder_summary_index (folder->summary, i);
@@ -638,7 +642,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			/*marked a message junk*/
 			move_to_junk (folder, info, ex);
 			camel_folder_summary_remove_uid (folder->summary, camel_message_info_uid(info));
-			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), ex);
+			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), NULL);
 			continue;
 		}
 
@@ -646,7 +650,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			/*message was marked as junk, now unjunk*/ 
 			move_to_mailbox (folder, info, ex);
 			camel_folder_summary_remove_uid (folder->summary, camel_message_info_uid(info));
-			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), ex);
+			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), NULL);
 			continue;
 		}
 
@@ -661,9 +665,11 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 				camel_message_info_free(info);
 				continue;
 			} else {
+				const char *uid;
+
 				gw_info->info.flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
 				gw_info->server_flags = gw_info->info.flags;
-				const char *uid = camel_message_info_uid (info);
+				uid = camel_message_info_uid (info);
 				if (diff.bits & CAMEL_MESSAGE_DELETED) {
 
 					/* In case a new message is READ and then deleted immediately */
@@ -679,7 +685,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 					}
 
 					if (g_list_length (deleted_items) == GROUPWISE_BULK_DELETE_LIMIT ) {
-						CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+						CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 
 						/* 
 							Sync up the READ changes before deleting the message. 
@@ -700,15 +706,15 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 						/* And now delete the messages */
 						status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
-						CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+						CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 						if (status == E_GW_CONNECTION_STATUS_OK) {
 							char *uid;
 							while (deleted_items) {
 								uid = (char *)deleted_items->data;
-								CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+								CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 								camel_folder_summary_remove_uid (folder->summary, uid);
-								camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
-								CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+								camel_data_cache_remove(gw_folder->cache, "cache", uid, NULL);
+								CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 								deleted_items = g_list_next (deleted_items);
 								count -= GROUPWISE_BULK_DELETE_LIMIT;
 								i -= GROUPWISE_BULK_DELETE_LIMIT;
@@ -725,24 +731,24 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		camel_message_info_free (info);
 	}
 	
-	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+	CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 
 	if (deleted_items) {
-		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		if (!strcmp (folder->full_name, "Trash")) {
 			status = e_gw_connection_purge_selected_items (cnc, deleted_items);
 		} else {
 			status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
 		}
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 			char *uid;
 			while (deleted_items) {
 				uid = (char *)deleted_items->data;
-				CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+				CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 				camel_folder_summary_remove_uid (folder->summary, uid);
-				camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
-				CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+				camel_data_cache_remove(gw_folder->cache, "cache", uid, NULL);
+				CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 				deleted_items = g_list_next (deleted_items);
 				count -= GROUPWISE_BULK_DELETE_LIMIT;
 				i -= GROUPWISE_BULK_DELETE_LIMIT;
@@ -752,31 +758,31 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	}
 
 	if (read_items) {
-		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		e_gw_connection_mark_read (cnc, read_items);
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 		g_list_free (read_items);
 	}
 
 	if (unread_items) {
-		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		e_gw_connection_mark_unread (cnc, unread_items);
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 		g_list_free (unread_items);
 	}
 
 	if (expunge) {
-		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		status = e_gw_connection_purge_deleted_items (cnc);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 			g_message ("Purged deleted items in %s", folder->name);
 		}
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 	}
 
-	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 	groupwise_sync_summary (folder, ex);
-	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 }
 
 CamelFolder *
@@ -948,9 +954,8 @@ static CamelSessionThreadOps update_ops = {
 static void
 groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 {
-	CamelGroupwiseSummary *summary;
+	CamelGroupwiseSummary *summary = (CamelGroupwiseSummary *) folder->summary;
 	CamelStoreInfo *si;
-	summary = (CamelGroupwiseSummary *) folder->summary;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
 	/*
 	 * Checking for the summary->time_string here since the first the a
@@ -1023,7 +1028,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		gw_folder->need_refresh = TRUE;
 	}
 
-	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 
 	if (!camel_groupwise_store_connected (gw_store, ex)) 
 		goto end1;
@@ -1058,9 +1063,17 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 
 	/*Get the New Items*/
 	if (!is_proxy) {
+		char *source;
+
+		if ( !strcmp (folder->full_name, RECEIVED) || !strcmp(folder->full_name, SENT) ) {
+			source = NULL;
+		} else {
+			source = "sent received";
+		}
+
 		status = e_gw_connection_get_quick_messages (cnc, container_id,
 				"peek id",
-				&t_str, "New", NULL, NULL, -1, &slist);
+				&t_str, "New", NULL, source, -1, &slist);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
 			goto end2;
@@ -1097,7 +1110,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 
 		status = e_gw_connection_get_quick_messages (cnc, container_id,
 				"peek id",
-				&t_str, "Modified", NULL, NULL, -1, &slist);
+				&t_str, "Modified", NULL, source, -1, &slist);
 
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
@@ -1117,6 +1130,9 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		for ( sl = slist ; sl != NULL; sl = sl->next) 
 			list = g_list_prepend (list, sl->data);
 
+		if (!check_all && slist && g_slist_length(slist) != 0)
+			check_all = TRUE;
+
 		g_slist_free (slist);
 		slist = NULL;
 
@@ -1130,7 +1146,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 	}
 
 
-	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+	CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 	is_locked = FALSE;
 
 	/*
@@ -1162,7 +1178,7 @@ end2:
 	g_free (container_id);
 end1:
 	if (is_locked)
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 	return;
 }
 
@@ -1175,7 +1191,6 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER(folder);
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 	EGwConnection *cnc = cnc_lookup (priv);
-	GSList *attach_list = NULL;
 	guint32 item_status, status_flags = 0;
 	CamelFolderChangeInfo *changes = NULL;
 	gboolean exists = FALSE;
@@ -1189,8 +1204,6 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 	gboolean is_proxy = folder->parent_store->flags & CAMEL_STORE_WRITE;
 
-	CAMEL_SERVICE_ASSERT_LOCKED (gw_store, connect_lock);
-
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
 	if (!container_id) {
@@ -1213,14 +1226,13 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 		char *temp_date = NULL;
 		const char *id;
 		GSList *recp_list = NULL;
-		status_flags = 0;
 		CamelStream *cache_stream, *t_cache_stream;
 		CamelMimeMessage *mail_msg = NULL;
-		gboolean is_sent_folder = FALSE;
 		const char *recurrence_key = NULL;
 		int rk;
 
 		exists = FALSE;
+		status_flags = 0;
 
 		if (uid_flag == FALSE) {
 			temp_item = (EGwItem *)item_list->data;
@@ -1309,6 +1321,12 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 				str = g_string_append (str, org->display_name);
 				str = g_string_append (str, " ");
 			}
+
+                        if (org->display_name[0] == '\0') { 
+
+				str = g_string_append (str, org->email);
+				str = g_string_append (str, " ");
+			}
 			if (org->email && org->email[0]) { 
 				g_string_append (str, "<");
 				str = g_string_append (str, org->email);
@@ -1381,19 +1399,17 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 		if (!strcmp (folder->full_name, "Junk Mail"))
 			continue;
 
-		if (!strcmp (folder->full_name, "Sent Items"))
-			is_sent_folder = TRUE;
 		/******************** Begine Caching ************************/
 		/* add to cache if its a new message*/
 		t_cache_stream  = camel_data_cache_get (gw_folder->cache, "cache", id, ex);
-		if (t_cache_stream && !is_sent_folder) {
+		if (t_cache_stream) {
 			camel_object_unref (t_cache_stream);
 
 			mail_msg = groupwise_folder_item_to_msg (folder, item, ex);
 			if (mail_msg)
 				camel_medium_set_header (CAMEL_MEDIUM (mail_msg), "X-Evolution-Source", groupwise_base_url_lookup (priv));
 
-			CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+			CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 			if ((cache_stream = camel_data_cache_add (gw_folder->cache, "cache", id, NULL))) {
 				if (camel_data_wrapper_write_to_stream ((CamelDataWrapper *) mail_msg, 	cache_stream) == -1 || camel_stream_flush (cache_stream) == -1)
 					camel_data_cache_remove (gw_folder->cache, "cache", id, NULL);
@@ -1401,7 +1417,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 			}
 
 			camel_object_unref (mail_msg);
-			CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+			CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 		}
 		/******************** Caching stuff ends *************************/
 		i++;
@@ -1419,7 +1435,6 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 {
 	CamelGroupwiseMessageInfo *mi = NULL;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
-	GSList *attach_list = NULL;
 	guint32 item_status, status_flags = 0;
 	CamelFolderChangeInfo *changes = NULL;
 	gboolean exists = FALSE;
@@ -1451,10 +1466,10 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 		char *temp_date = NULL;
 		const char *id;
 		GSList *recp_list = NULL;
-		status_flags = 0;
 		const char *recurrence_key = NULL;
 		int rk;
 
+		status_flags = 0;
 		id = e_gw_item_get_id (item);
 
 		mi = (CamelGroupwiseMessageInfo *)camel_folder_summary_uid (folder->summary, id);
@@ -1524,6 +1539,13 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 				str = g_string_append (str, org->display_name);
 				str = g_string_append (str, " ");
 			}
+
+                        if (org->display_name[0] == '\0') { 
+				
+                                str = g_string_append (str, org->email);
+				str = g_string_append (str, " ");
+			}
+
 			if (org->email && org->email[0]) { 
 				g_string_append (str, "<");
 				str = g_string_append (str, org->email);
@@ -1611,7 +1633,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	EGwItemType type;
 	EGwConnectionStatus status;
 	EGwConnection *cnc;
-	CamelMultipart *multipart;
+	CamelMultipart *multipart = NULL;
 	int errno;
 	char *body = NULL;
 	int body_len = 0;
@@ -1911,11 +1933,11 @@ gw_update_all_items (CamelFolder *folder, GList *item_list, CamelException *ex)
 		}
 
 		if (!temp) {
-			CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+			CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 			camel_folder_summary_remove_uid (folder->summary, info->uid);
-			camel_data_cache_remove (gw_folder->cache, "cache", info->uid, ex);
+			camel_data_cache_remove (gw_folder->cache, "cache", info->uid, NULL);
 			camel_folder_change_info_remove_uid (changes, info->uid);
-			CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+			CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 		} else { 
 			item_list = g_list_delete_link (item_list, temp);
 		}
@@ -1927,9 +1949,9 @@ gw_update_all_items (CamelFolder *folder, GList *item_list, CamelException *ex)
 	if (item_list) {
 		CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store);
 
-		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		gw_update_cache (folder, item_list, ex, TRUE);	
-		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
 
 		g_list_foreach (item_list, (GFunc)g_free, NULL);	
 		g_list_free (item_list);
@@ -1947,7 +1969,7 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	CamelGroupwiseStore *gw_store= CAMEL_GROUPWISE_STORE(folder->parent_store);
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	CamelOfflineStore *offline = (CamelOfflineStore *) folder->parent_store;
-	EGwConnectionStatus status;
+	EGwConnectionStatus status = { 0, };
 	EGwConnection *cnc;
 	EGwItem *item;
 	char *id;
@@ -1970,7 +1992,7 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	}
 	cnc = cnc_lookup (priv);
 
-	CAMEL_SERVICE_LOCK (folder->parent_store, connect_lock);
+	CAMEL_SERVICE_REC_LOCK (folder->parent_store, connect_lock);
 	/*Get the container id*/
 	container_id = camel_groupwise_store_container_id_lookup (gw_store, folder->full_name) ;
 	
@@ -1994,7 +2016,7 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 
 		if (appended_uid)
 			*appended_uid = NULL;
-		CAMEL_SERVICE_UNLOCK (folder->parent_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (folder->parent_store, connect_lock);
 		return;
 	}
 
@@ -2007,14 +2029,14 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 		if (appended_uid)
 			*appended_uid = NULL;
 
-		CAMEL_SERVICE_UNLOCK (folder->parent_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (folder->parent_store, connect_lock);
 		return;
 	}
 
 	if (appended_uid)
 		*appended_uid = g_strdup (id);	
 	g_free (id);
-	CAMEL_SERVICE_UNLOCK (folder->parent_store, connect_lock);
+	CAMEL_SERVICE_REC_UNLOCK (folder->parent_store, connect_lock);
 }
 
 static int
@@ -2046,10 +2068,12 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	EGwConnectionStatus status;
 	EGwConnection *cnc;
+	CamelFolderChangeInfo *changes = NULL;
 
 	count = camel_folder_summary_count (destination->summary);
 	qsort (uids->pdata, uids->len, sizeof (void *), uid_compar);
 
+	changes = camel_folder_change_info_new ();
 	while (index < uids->len) {
 		item_ids = g_list_append (item_ids, g_ptr_array_index (uids, index));
 		index ++;
@@ -2064,7 +2088,7 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		source_container_id = NULL;
 	dest_container_id = camel_groupwise_store_container_id_lookup (gw_store, destination->full_name) ;
 
-	CAMEL_SERVICE_LOCK (source->parent_store, connect_lock);
+	CAMEL_SERVICE_REC_LOCK (source->parent_store, connect_lock);
 	/* check for offline operation */
 	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		CamelGroupwiseJournal *journal = (CamelGroupwiseJournal *) ((CamelGroupwiseFolder *) destination)->journal;
@@ -2087,11 +2111,19 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			if (camel_exception_is_set (ex))
 				break;
 
-			if (delete_originals)
-				camel_folder_delete_message(source, camel_message_info_uid (info));
+			if (delete_originals) { 
+				if ( !strcmp(source->full_name, SENT) ) {
+					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+							_("This message is not available in offline mode."));
+
+				} else {
+					camel_folder_summary_remove_uid (source->summary, uids->pdata[index]);
+					camel_folder_change_info_remove_uid (changes, uids->pdata[index]);
+				}
+			}
 		}
 
-		CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (source->parent_store, connect_lock);
 		return;
 	}
 
@@ -2128,13 +2160,14 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 					*/
 
 					wrapper = g_list_prepend (wrapper, (char *)uid);
-					CAMEL_SERVICE_LOCK (source->parent_store, connect_lock);
+					CAMEL_SERVICE_REC_LOCK (source->parent_store, connect_lock);
 					e_gw_connection_mark_read (cnc, wrapper);
-					CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+					CAMEL_SERVICE_REC_UNLOCK (source->parent_store, connect_lock);
 					g_list_free (wrapper);
 					wrapper = NULL;
 				}
 
+				
 				/* A User may mark a message as Unread and then immediately move it to
 				some other folder. The following piece of code take care of such scenario.
 				
@@ -2144,32 +2177,51 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 
 				if (unset_flags.bits & CAMEL_MESSAGE_SEEN) {
 					wrapper = g_list_prepend (wrapper, (char *)uid);
-					CAMEL_SERVICE_LOCK (source->parent_store, connect_lock);
+					CAMEL_SERVICE_REC_LOCK (source->parent_store, connect_lock);
 					e_gw_connection_mark_unread (cnc, wrapper);
-					CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+					CAMEL_SERVICE_REC_UNLOCK (source->parent_store, connect_lock);
 					g_list_free (wrapper);
 					wrapper = NULL;
 				}
 			}
 		}
 
-		if (delete_originals) 
-			status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
-					dest_container_id, source_container_id);
-		else
+		if (delete_originals) {
+			if (strcmp(source->full_name, "Sent Items")) {
+				status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
+						dest_container_id, source_container_id);
+			} else {
+				char *container_id = NULL;
+				container_id = e_gw_connection_get_container_id (cnc, "Mailbox");
+				status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
+						dest_container_id, container_id);
+				g_free (container_id);
+			}
+
+		} else
 			status = e_gw_connection_move_item (cnc, (const char *)uids->pdata[index], 
 					dest_container_id, NULL);
+
 		if (status == E_GW_CONNECTION_STATUS_OK) {
-			if (delete_originals) 
-				camel_folder_delete_message(source, uids->pdata[index]);
+			if (delete_originals) { 
+				if ( !strcmp(source->full_name, SENT) ) {
+					camel_folder_delete_message(source, uids->pdata[index]);
+				} else {
+					camel_folder_summary_remove_uid (source->summary, uids->pdata[index]);
+					camel_folder_change_info_remove_uid (changes, uids->pdata[index]);
+				}
+			}
 		} else {
 			g_warning ("Warning!! Could not move item : %s\n", (char *)uids->pdata[index]);
 		}
 		index ++;
 	}
+
+	camel_object_trigger_event (source, "folder_changed", changes);
+	camel_folder_change_info_free (changes);
+
 	/* Refresh the destination folder, if its not refreshed already */
-	if (gw_store->current_folder != destination || 
-			camel_folder_summary_count (destination->summary) == count)
+	if (gw_store->current_folder != destination )
 		camel_folder_refresh_info (destination, ex);
 
 	camel_folder_summary_touch (source->summary);
@@ -2177,7 +2229,7 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 
 	gw_store->current_folder = source;
 
-	CAMEL_SERVICE_UNLOCK (source->parent_store, connect_lock);
+	CAMEL_SERVICE_REC_UNLOCK (source->parent_store, connect_lock);
 }
 
 static void
@@ -2203,7 +2255,7 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 		return;
 
 	if (!strcmp (folder->full_name, "Trash")) {
-		CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (groupwise_store, connect_lock);
 		status = e_gw_connection_purge_deleted_items (cnc);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 			camel_folder_freeze (folder);
@@ -2211,7 +2263,7 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 			camel_folder_thaw (folder);
 		} else
 			g_warning ("Could not Empty Trash\n");
-		CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (groupwise_store, connect_lock);
 		return;
 	}
 
@@ -2234,18 +2286,18 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 			}
 			if (g_list_length (deleted_items) == GROUPWISE_BULK_DELETE_LIMIT ) {
 				/* Read the FIXME below */
-				CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
+				CAMEL_SERVICE_REC_LOCK (groupwise_store, connect_lock);
 				status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
-				CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
+				CAMEL_SERVICE_REC_UNLOCK (groupwise_store, connect_lock);
 				if (status == E_GW_CONNECTION_STATUS_OK) {
 					char *uid;
 					while (deleted_items) {
 						uid = (char *)deleted_items->data;
-						CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+						CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 						camel_folder_change_info_remove_uid (changes, uid);
 						camel_folder_summary_remove_uid (folder->summary, uid);
-						camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
-						CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+						camel_data_cache_remove(gw_folder->cache, "cache", uid, NULL);
+						CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 						deleted_items = g_list_next (deleted_items);
 						max -= GROUPWISE_BULK_DELETE_LIMIT;
 						i -= GROUPWISE_BULK_DELETE_LIMIT;
@@ -2259,18 +2311,18 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 
 	if (deleted_items) {
 		/* FIXME: Put these in a function and reuse it inside the above loop, here and in groupwise_sync*/
-		CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
+		CAMEL_SERVICE_REC_LOCK (groupwise_store, connect_lock);
 		status = e_gw_connection_remove_items (cnc, container_id, deleted_items);
-		CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
+		CAMEL_SERVICE_REC_UNLOCK (groupwise_store, connect_lock);
 		if (status == E_GW_CONNECTION_STATUS_OK) {
 			char *uid;
 			while (deleted_items) {
 				uid = (char *)deleted_items->data;
-				CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
+				CAMEL_GROUPWISE_FOLDER_REC_LOCK (folder, cache_lock);
 				camel_folder_change_info_remove_uid (changes, uid);
 				camel_folder_summary_remove_uid (folder->summary, uid);
-				camel_data_cache_remove(gw_folder->cache, "cache", uid, ex);
-				CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+				camel_data_cache_remove(gw_folder->cache, "cache", uid, NULL);
+				CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 				deleted_items = g_list_next (deleted_items);
 			}
 		}
@@ -2321,9 +2373,10 @@ camel_groupwise_folder_init (gpointer object, gpointer klass)
 	gw_folder->priv = g_malloc0 (sizeof(*gw_folder->priv));
 
 #ifdef ENABLE_THREADS
-	gw_folder->priv->search_lock = e_mutex_new(E_MUTEX_SIMPLE);
-	gw_folder->priv->cache_lock = e_mutex_new(E_MUTEX_REC);
+	g_static_mutex_init(&gw_folder->priv->search_lock);
+	g_static_rec_mutex_init(&gw_folder->priv->cache_lock);
 #endif 
+
 	gw_folder->need_rescan = TRUE;
 }
 
