@@ -209,10 +209,10 @@ static gboolean
 xml_set_bool (xmlNodePtr node, const char *name, gboolean *val)
 {
 	gboolean bool;
-	char *buf;
+	xmlChar *buf;
 
-	if ((buf = xmlGetProp (node, name))) {
-		bool = (!strcmp (buf, "true") || !strcmp (buf, "yes"));
+	if ((buf = xmlGetProp (node, (xmlChar*)name))) {
+		bool = (!strcmp ((char*)buf, "true") || !strcmp ((char*)buf, "yes"));
 		xmlFree (buf);
 
 		if (bool != *val) {
@@ -228,10 +228,10 @@ static gboolean
 xml_set_int (xmlNodePtr node, const char *name, int *val)
 {
 	int number;
-	char *buf;
+	xmlChar *buf;
 
-	if ((buf = xmlGetProp (node, name))) {
-		number = strtol (buf, NULL, 10);
+	if ((buf = xmlGetProp (node, (xmlChar*)name))) {
+		number = strtol ((char*)buf, NULL, 10);
 		xmlFree (buf);
 
 		if (number != *val) {
@@ -246,10 +246,10 @@ xml_set_int (xmlNodePtr node, const char *name, int *val)
 static gboolean
 xml_set_prop (xmlNodePtr node, const char *name, char **val)
 {
-	char *buf;
+	xmlChar *buf;
 	int res;
 
-	buf = xmlGetProp (node, name);
+	buf = xmlGetProp (node, (xmlChar*)name);
 	if (buf == NULL) {
 		res = (*val != NULL);
 		if (res) {
@@ -257,10 +257,10 @@ xml_set_prop (xmlNodePtr node, const char *name, char **val)
 			*val = NULL;
 		}
 	} else {
-		res = *val == NULL || strcmp(*val, buf) != 0;
+		res = *val == NULL || strcmp(*val, (char*)buf) != 0;
 		if (res) {
 			g_free(*val);
-			*val = g_strdup(buf);
+			*val = g_strdup((char*)buf);
 		}
 		xmlFree(buf);
 	}
@@ -269,17 +269,17 @@ xml_set_prop (xmlNodePtr node, const char *name, char **val)
 }
 
 static EAccountReceiptPolicy
-str_to_receipt_policy (const char *str)
+str_to_receipt_policy (const xmlChar *str)
 {
-	if (!strcmp (str, "ask"))
+	if (!strcmp ((char*)str, "ask"))
 		return E_ACCOUNT_RECEIPT_ASK;
-	if (!strcmp (str, "always"))
+	if (!strcmp ((char*)str, "always"))
 		return E_ACCOUNT_RECEIPT_ALWAYS;
 
 	return E_ACCOUNT_RECEIPT_NEVER;
 }
 
-static char*
+static xmlChar*
 receipt_policy_to_str (EAccountReceiptPolicy val)
 {
 	char *ret = NULL;
@@ -296,16 +296,16 @@ receipt_policy_to_str (EAccountReceiptPolicy val)
 		break;
 	}
 
-	return ret;
+	return (xmlChar*)ret;
 }
 
 static gboolean
 xml_set_receipt_policy (xmlNodePtr node, const char *name, EAccountReceiptPolicy *val)
 {
 	EAccountReceiptPolicy new_val;
-	char *buf;
+	xmlChar *buf;
 
-	if ((buf = xmlGetProp (node, name))) {
+	if ((buf = xmlGetProp (node, (xmlChar*)name))) {
 		new_val = str_to_receipt_policy (buf);
 		xmlFree (buf);
 
@@ -321,7 +321,7 @@ xml_set_receipt_policy (xmlNodePtr node, const char *name, EAccountReceiptPolicy
 static gboolean
 xml_set_content (xmlNodePtr node, char **val)
 {
-	char *buf;
+	xmlChar *buf;
 	int res;
 
 	buf = xmlNodeGetContent(node);
@@ -332,10 +332,10 @@ xml_set_content (xmlNodePtr node, char **val)
 			*val = NULL;
 		}
 	} else {
-		res = *val == NULL || strcmp(*val, buf) != 0;
+		res = *val == NULL || strcmp(*val, (char*)buf) != 0;
 		if (res) {
 			g_free(*val);
-			*val = g_strdup(buf);
+			*val = g_strdup((char*)buf);
 		}
 		xmlFree(buf);
 	}
@@ -349,15 +349,15 @@ xml_set_identity (xmlNodePtr node, EAccountIdentity *id)
 	gboolean changed = FALSE;
 
 	for (node = node->children; node; node = node->next) {
-		if (!strcmp (node->name, "name"))
+		if (!strcmp ((char*)node->name, "name"))
 			changed |= xml_set_content (node, &id->name);
-		else if (!strcmp (node->name, "addr-spec"))
+		else if (!strcmp ((char*)node->name, "addr-spec"))
 			changed |= xml_set_content (node, &id->address);
-		else if (!strcmp (node->name, "reply-to"))
+		else if (!strcmp ((char*)node->name, "reply-to"))
 			changed |= xml_set_content (node, &id->reply_to);
-		else if (!strcmp (node->name, "organization"))
+		else if (!strcmp ((char*)node->name, "organization"))
 			changed |= xml_set_content (node, &id->organization);
-		else if (!strcmp (node->name, "signature")) {
+		else if (!strcmp ((char*)node->name, "signature")) {
 			changed |= xml_set_prop (node, "uid", &id->sig_uid);
 			if (!id->sig_uid) {
 
@@ -400,7 +400,7 @@ xml_set_service (xmlNodePtr node, EAccountService *service)
 	}
 
 	for (node = node->children; node; node = node->next) {
-		if (!strcmp (node->name, "url")) {
+		if (!strcmp ((char*)node->name, "url")) {
 			changed |= xml_set_content (node, &service->url);
 			break;
 		}
@@ -426,11 +426,11 @@ e_account_set_from_xml (EAccount *account, const char *xml)
 	xmlDocPtr doc;
 	gboolean changed = FALSE;
 
-	if (!(doc = xmlParseDoc ((char *)xml)))
+	if (!(doc = xmlParseDoc ((xmlChar*)xml)))
 		return FALSE;
 
 	node = doc->children;
-	if (strcmp (node->name, "account") != 0) {
+	if (strcmp ((char*)node->name, "account") != 0) {
 		xmlFreeDoc (doc);
 		return FALSE;
 	}
@@ -442,25 +442,25 @@ e_account_set_from_xml (EAccount *account, const char *xml)
 	changed |= xml_set_bool (node, "enabled", &account->enabled);
 
 	for (node = node->children; node; node = node->next) {
-		if (!strcmp (node->name, "identity")) {
+		if (!strcmp ((char*)node->name, "identity")) {
 			changed |= xml_set_identity (node, account->id);
-		} else if (!strcmp (node->name, "source")) {
+		} else if (!strcmp ((char*)node->name, "source")) {
 			changed |= xml_set_service (node, account->source);
-		} else if (!strcmp (node->name, "transport")) {
+		} else if (!strcmp ((char*)node->name, "transport")) {
 			changed |= xml_set_service (node, account->transport);
-		} else if (!strcmp (node->name, "drafts-folder")) {
+		} else if (!strcmp ((char*)node->name, "drafts-folder")) {
 			changed |= xml_set_content (node, &account->drafts_folder_uri);
-		} else if (!strcmp (node->name, "sent-folder")) {
+		} else if (!strcmp ((char*)node->name, "sent-folder")) {
 			changed |= xml_set_content (node, &account->sent_folder_uri);
-		} else if (!strcmp (node->name, "auto-cc")) {
+		} else if (!strcmp ((char*)node->name, "auto-cc")) {
 			changed |= xml_set_bool (node, "always", &account->always_cc);
 			changed |= xml_set_content (node, &account->cc_addrs);
-		} else if (!strcmp (node->name, "auto-bcc")) {
+		} else if (!strcmp ((char*)node->name, "auto-bcc")) {
 			changed |= xml_set_bool (node, "always", &account->always_bcc);
 			changed |= xml_set_content (node, &account->bcc_addrs);
-		} else if (!strcmp (node->name, "receipt-policy")) {
+		} else if (!strcmp ((char*)node->name, "receipt-policy")) {
 			changed |= xml_set_receipt_policy (node, "policy", &account->receipt_policy);
-		} else if (!strcmp (node->name, "pgp")) {
+		} else if (!strcmp ((char*)node->name, "pgp")) {
 			changed |= xml_set_bool (node, "encrypt-to-self", &account->pgp_encrypt_to_self);
 			changed |= xml_set_bool (node, "always-trust", &account->pgp_always_trust);
 			changed |= xml_set_bool (node, "always-sign", &account->pgp_always_sign);
@@ -468,31 +468,31 @@ e_account_set_from_xml (EAccount *account, const char *xml)
 
 			if (node->children) {
 				for (cur = node->children; cur; cur = cur->next) {
-					if (!strcmp (cur->name, "key-id")) {
+					if (!strcmp ((char*)cur->name, "key-id")) {
 						changed |= xml_set_content (cur, &account->pgp_key);
 						break;
 					}
 				}
 			}
-		} else if (!strcmp (node->name, "smime")) {
+		} else if (!strcmp ((char*)node->name, "smime")) {
 			changed |= xml_set_bool (node, "sign-default", &account->smime_sign_default);
 			changed |= xml_set_bool (node, "encrypt-to-self", &account->smime_encrypt_to_self);
 			changed |= xml_set_bool (node, "encrypt-default", &account->smime_encrypt_default);
 
 			if (node->children) {
 				for (cur = node->children; cur; cur = cur->next) {
-					if (!strcmp (cur->name, "sign-key-id")) {
+					if (!strcmp ((char*)cur->name, "sign-key-id")) {
 						changed |= xml_set_content (cur, &account->smime_sign_key);
-					} else if (!strcmp (cur->name, "encrypt-key-id")) {
+					} else if (!strcmp ((char*)cur->name, "encrypt-key-id")) {
 						changed |= xml_set_content (cur, &account->smime_encrypt_key);
 						break;
 					}
 				}
 			}
-		} else if (!strcmp (node->name, "proxy")) {
+		} else if (!strcmp ((char*)node->name, "proxy")) {
 			if (node->children) {
 				for (cur = node->children; cur; cur = cur->next) {
-					if (!strcmp (cur->name, "parent-uid")) {
+					if (!strcmp ((char*)cur->name, "parent-uid")) {
 						changed |= xml_set_content (cur, &account->parent_uid);
 						break;
 					}
@@ -597,78 +597,78 @@ e_account_to_xml (EAccount *account)
 	xmlDocPtr doc;
 	int n;
 
-	doc = xmlNewDoc ("1.0");
+	doc = xmlNewDoc ((xmlChar*)"1.0");
 
-	root = xmlNewDocNode (doc, NULL, "account", NULL);
+	root = xmlNewDocNode (doc, NULL, (xmlChar*)"account", NULL);
 	xmlDocSetRootElement (doc, root);
 
-	xmlSetProp (root, "name", account->name);
-	xmlSetProp (root, "uid", account->uid);
-	xmlSetProp (root, "enabled", account->enabled ? "true" : "false");
+	xmlSetProp (root, (xmlChar*)"name", (xmlChar*)account->name);
+	xmlSetProp (root, (xmlChar*)"uid", (xmlChar*)account->uid);
+	xmlSetProp (root, (xmlChar*)"enabled", (xmlChar*)(account->enabled ? "true" : "false"));
 
-	id = xmlNewChild (root, NULL, "identity", NULL);
+	id = xmlNewChild (root, NULL, (xmlChar*)"identity", NULL);
 	if (account->id->name)
-		xmlNewTextChild (id, NULL, "name", account->id->name);
+		xmlNewTextChild (id, NULL, (xmlChar*)"name", (xmlChar*)account->id->name);
 	if (account->id->address)
-		xmlNewTextChild (id, NULL, "addr-spec", account->id->address);
+		xmlNewTextChild (id, NULL, (xmlChar*)"addr-spec", (xmlChar*)account->id->address);
 	if (account->id->reply_to)
-		xmlNewTextChild (id, NULL, "reply-to", account->id->reply_to);
+		xmlNewTextChild (id, NULL, (xmlChar*)"reply-to", (xmlChar*)account->id->reply_to);
 	if (account->id->organization)
-		xmlNewTextChild (id, NULL, "organization", account->id->organization);
+		xmlNewTextChild (id, NULL, (xmlChar*)"organization", (xmlChar*)account->id->organization);
 
-	node = xmlNewChild (id, NULL, "signature",NULL);
-	xmlSetProp (node, "uid", account->id->sig_uid);
+	node = xmlNewChild (id, NULL, (xmlChar*)"signature",NULL);
+	xmlSetProp (node, (xmlChar*)"uid", (xmlChar*)account->id->sig_uid);
 
-	src = xmlNewChild (root, NULL, "source", NULL);
-	xmlSetProp (src, "save-passwd", account->source->save_passwd ? "true" : "false");
-	xmlSetProp (src, "keep-on-server", account->source->keep_on_server ? "true" : "false");
-	xmlSetProp (src, "auto-check", account->source->auto_check ? "true" : "false");
+	src = xmlNewChild (root, NULL, (xmlChar*)"source", NULL);
+	xmlSetProp (src, (xmlChar*)"save-passwd", (xmlChar*)(account->source->save_passwd ? "true" : "false"));
+	xmlSetProp (src, (xmlChar*)"keep-on-server", (xmlChar*)(account->source->keep_on_server ? "true" : "false"));
+	xmlSetProp (src, (xmlChar*)"auto-check", (xmlChar*)(account->source->auto_check ? "true" : "false"));
 	sprintf (buf, "%d", account->source->auto_check_time);
-	xmlSetProp (src, "auto-check-timeout", buf);
+	xmlSetProp (src, (xmlChar*)"auto-check-timeout", (xmlChar*)buf);
 	if (account->source->url)
-		xmlNewTextChild (src, NULL, "url", account->source->url);
+		xmlNewTextChild (src, NULL, (xmlChar*)"url", (xmlChar*)account->source->url);
 
-	xport = xmlNewChild (root, NULL, "transport", NULL);
-	xmlSetProp (xport, "save-passwd", account->transport->save_passwd ? "true" : "false");
+	xport = xmlNewChild (root, NULL, (xmlChar*)"transport", NULL);
+	xmlSetProp (xport, (xmlChar*)"save-passwd", (xmlChar*)(account->transport->save_passwd ? "true" : "false"));
 	if (account->transport->url)
-		xmlNewTextChild (xport, NULL, "url", account->transport->url);
+		xmlNewTextChild (xport, NULL, (xmlChar*)"url", (xmlChar*)account->transport->url);
 
-	xmlNewTextChild (root, NULL, "drafts-folder", account->drafts_folder_uri);
-	xmlNewTextChild (root, NULL, "sent-folder", account->sent_folder_uri);
+	xmlNewTextChild (root, NULL, (xmlChar*)"drafts-folder", (xmlChar*)account->drafts_folder_uri);
+	xmlNewTextChild (root, NULL, (xmlChar*)"sent-folder", (xmlChar*)account->sent_folder_uri);
 
-	node = xmlNewChild (root, NULL, "auto-cc", NULL);
-	xmlSetProp (node, "always", account->always_cc ? "true" : "false");
+	node = xmlNewChild (root, NULL, (xmlChar*)"auto-cc", NULL);
+	xmlSetProp (node, (xmlChar*)"always", (xmlChar*)(account->always_cc ? "true" : "false"));
 	if (account->cc_addrs)
-		xmlNewTextChild (node, NULL, "recipients", account->cc_addrs);
+		xmlNewTextChild (node, NULL, (xmlChar*)"recipients", (xmlChar*)account->cc_addrs);
 
-	node = xmlNewChild (root, NULL, "auto-bcc", NULL);
-	xmlSetProp (node, "always", account->always_bcc ? "true" : "false");
+	node = xmlNewChild (root, NULL, (xmlChar*)"auto-bcc", NULL);
+	xmlSetProp (node, (xmlChar*)"always", (xmlChar*)(account->always_bcc ? "true" : "false"));
 	if (account->bcc_addrs)
-		xmlNewTextChild (node, NULL, "recipients", account->bcc_addrs);
+		xmlNewTextChild (node, NULL, (xmlChar*)"recipients", (xmlChar*)account->bcc_addrs);
 
-	node = xmlNewChild (root, NULL, "receipt-policy", NULL);
-	xmlSetProp (node, "policy", receipt_policy_to_str (account->receipt_policy));
+	node = xmlNewChild (root, NULL, (xmlChar*)"receipt-policy", NULL);
+	xmlSetProp (node, (xmlChar*)"policy", receipt_policy_to_str (account->receipt_policy));
 	
-	node = xmlNewChild (root, NULL, "pgp", NULL);
-	xmlSetProp (node, "encrypt-to-self", account->pgp_encrypt_to_self ? "true" : "false");
-	xmlSetProp (node, "always-trust", account->pgp_always_trust ? "true" : "false");
-	xmlSetProp (node, "always-sign", account->pgp_always_sign ? "true" : "false");
-	xmlSetProp (node, "no-imip-sign", account->pgp_no_imip_sign ? "true" : "false");
+	node = xmlNewChild (root, NULL, (xmlChar*)"pgp", NULL);
+	xmlSetProp (node, (xmlChar*)"encrypt-to-self", (xmlChar*)(account->pgp_encrypt_to_self ? "true" : "false"));
+	xmlSetProp (node, (xmlChar*)"always-trust", (xmlChar*)(account->pgp_always_trust ? "true" : "false"));
+	xmlSetProp (node, (xmlChar*)"always-sign", (xmlChar*)(account->pgp_always_sign ? "true" : "false"));
+	xmlSetProp (node, (xmlChar*)"no-imip-sign", (xmlChar*)(account->pgp_no_imip_sign ? "true" : "false"));
 	if (account->pgp_key)
-		xmlNewTextChild (node, NULL, "key-id", account->pgp_key);
+		xmlNewTextChild (node, NULL, (xmlChar*)"key-id", (xmlChar*)account->pgp_key);
 
-	node = xmlNewChild (root, NULL, "smime", NULL);
-	xmlSetProp (node, "sign-default", account->smime_sign_default ? "true" : "false");
-	xmlSetProp (node, "encrypt-default", account->smime_encrypt_default ? "true" : "false");
-	xmlSetProp (node, "encrypt-to-self", account->smime_encrypt_to_self ? "true" : "false");
+	node = xmlNewChild (root, NULL, (xmlChar*)"smime", NULL);
+	xmlSetProp (node, (xmlChar*)"sign-default", (xmlChar*)(account->smime_sign_default ? "true" : "false"));
+	xmlSetProp (node, (xmlChar*)"encrypt-default", (xmlChar*)(account->smime_encrypt_default ? "true" : "false"));
+	xmlSetProp (node, (xmlChar*)"encrypt-to-self", (xmlChar*)(account->smime_encrypt_to_self ? "true" : "false"));
 	if (account->smime_sign_key)
-		xmlNewTextChild (node, NULL, "sign-key-id", account->smime_sign_key);
+		xmlNewTextChild (node, NULL, (xmlChar*)"sign-key-id", (xmlChar*)account->smime_sign_key);
 	if (account->smime_encrypt_key)
-		xmlNewTextChild (node, NULL, "encrypt-key-id", account->smime_encrypt_key);
+		xmlNewTextChild (node, NULL, (xmlChar*)"encrypt-key-id", (xmlChar*)account->smime_encrypt_key);
 
 	if (account->parent_uid) {
-		node = xmlNewChild (root, NULL, "proxy", NULL);
-		xmlNewTextChild (node, NULL, "parent-uid", account->parent_uid);
+		node = xmlNewChild (root, NULL, (xmlChar*)"proxy", NULL);
+		xmlNewTextChild (node, NULL, (xmlChar*)"parent-uid", (xmlChar*)account->parent_uid);
 	}	
 
 	xmlDocDumpMemory (doc, &xmlbuf, &n);
@@ -699,11 +699,11 @@ e_account_uid_from_xml (const char *xml)
 	xmlDocPtr doc;
 	char *uid = NULL;
 
-	if (!(doc = xmlParseDoc ((char *)xml)))
+	if (!(doc = xmlParseDoc ((xmlChar *)xml)))
 		return NULL;
 
 	node = doc->children;
-	if (strcmp (node->name, "account") != 0) {
+	if (strcmp ((char*)node->name, "account") != 0) {
 		xmlFreeDoc (doc);
 		return NULL;
 	}
