@@ -45,47 +45,7 @@
 int
 e_util_mkdir_hier (const char *path, mode_t mode)
 {
-#if GLIB_CHECK_VERSION(2,8,0)
 	return g_mkdir_with_parents (path, mode);
-#else
-        char *copy, *p;
-                                                                                
-        if (g_path_is_absolute (path)) {
-                p = copy = g_strdup (path);
-        } else {
-                gchar *current_dir = g_get_current_dir();
-                p = copy = g_build_filename (current_dir, path, NULL);
-		g_free (current_dir);
-        }
-                                                                                
-	p = (char *)g_path_skip_root (p);                                                                       
-        do {
-                char *p1 = strchr (p, '/');
-#ifdef G_OS_WIN32
-		{
-			char *p2 = strchr (p, '\\');
-			if (p1 == NULL ||
-			    (p2 != NULL && p2 < p1))
-				p1 = p2;
-		}
-#endif
-		p = p1;
-		if (p)
-			*p = '\0';
-		if (!g_file_test (copy, G_FILE_TEST_IS_DIR)) {
-			if (g_mkdir (copy, mode) == -1) {
-				g_free (copy);
-				return -1;
-			}
-		}
-		if (p) {
-			*p++ = '/';
-		}
-        } while (p);
-                                                                                
-        g_free (copy);
-        return 0;
-#endif
 }
 
 /**
@@ -158,7 +118,7 @@ e_util_utf8_strstrcase (const gchar *haystack, const gchar *needle)
         gunichar *nuni;
         gunichar unival;
         gint nlen;
-        const guchar *o, *p;
+        const gchar *o, *p;
 
         if (haystack == NULL) return NULL;
         if (needle == NULL) return NULL;
@@ -205,7 +165,7 @@ stripped_char (gunichar ch)
 {
         gunichar *decomp, retval;
         GUnicodeType utype;
-        gint dlen;
+        gsize dlen;
 
         utype = g_unichar_type (ch);
 
@@ -250,7 +210,7 @@ e_util_utf8_strstrcasedecomp (const gchar *haystack, const gchar *needle)
         gunichar *nuni;
         gunichar unival;
         gint nlen;
-        const guchar *o, *p;
+        const gchar *o, *p;
 
         if (haystack == NULL) return NULL;
         if (needle == NULL) return NULL;
@@ -335,11 +295,11 @@ e_util_utf8_strcasecmp (const gchar *s1, const gchar *s2)
  **/
 size_t e_strftime(char *s, size_t max, const char *fmt, const struct tm *tm)
 {
+	size_t ret;
 #ifdef HAVE_LKSTRFTIME
-	return strftime(s, max, fmt, tm);
+	ret = strftime(s, max, fmt, tm);
 #else
 	char *c, *ffmt, *ff;
-	size_t ret;
 
 	ffmt = g_strdup(fmt);
 	ff = ffmt;
@@ -365,8 +325,10 @@ size_t e_strftime(char *s, size_t max, const char *fmt, const struct tm *tm)
 
 	ret = strftime(s, max, ffmt, tm);
 	g_free(ffmt);
-	return ret;
 #endif
+	if (ret == 0 && max > 0)
+		s[0] = '\0';
+	return ret;
 }
 
 /** 
@@ -493,9 +455,9 @@ e_filename_make_safe (gchar *string)
 	gchar *p, *ts;
 	gunichar c;
 #ifdef G_OS_WIN32
-	const char *unsafe_chars = " /'\"`&();|<>$%{}!\\:*?";
+	const char *unsafe_chars = " /'\"`&();|<>$%{}!\\:*?#";
 #else
-	const char *unsafe_chars = " /'\"`&();|<>$%{}!";
+	const char *unsafe_chars = " /'\"`&();|<>$%{}!#";
 #endif	
 	
 	g_return_if_fail (string != NULL);
