@@ -934,7 +934,8 @@ __strptime_internal (rp, fmt, tm, decided, era_cnt LOCALE_PARAM)
 #elif defined (G_OS_WIN32)
 	  if (*decided != raw)
 	    {
-	      if (!recursive (get_locale_string (LOCALE_STIMEFORMAT)))
+	      const char *posix_t_fmt = translate_picture (get_locale_string (LOCALE_STIMEFORMAT));
+	      if (!recursive (posix_t_fmt))
 		{
 		  if (*decided == loc)
 		    return NULL;
@@ -943,7 +944,7 @@ __strptime_internal (rp, fmt, tm, decided, era_cnt LOCALE_PARAM)
 		}
 	      else
 		{
-		  if (strcmp (get_locale_string (LOCALE_STIMEFORMAT), HERE_T_FMT))
+		  if (strcmp (posix_t_fmt, HERE_T_FMT))
 		    *decided = loc;
 		  break;
 		}
@@ -1495,6 +1496,9 @@ parse_with_strptime (const char *value, struct tm *result, const char **formats,
 	locale_str = g_locale_from_utf8 (value, -1, NULL, NULL, NULL);
 	pos = (const char *) locale_str;
 
+	if (!locale_str)
+		return E_TIME_PARSE_INVALID;
+	
 	/* Skip whitespace */
 	while (n = (int)((unsigned char)*pos), isspace (n) != 0)
 		pos++;
